@@ -10,6 +10,17 @@ class DashboardQueryInline(admin.TabularInline):
     model = DashboardQuery
     extra = 1
 
+    def has_change_permission(self, request, obj=None):
+        if obj is None:
+            return True
+        return obj.user_can_edit(request.user)
+
+    def get_readonly_fields(self, request, obj=None):
+        if not request.user.has_perm("django_sql_dashboard.execute_sql"):
+            return ("sql",)
+        else:
+            return tuple()
+
 
 @admin.register(Dashboard)
 class DashboardAdmin(admin.ModelAdmin):
@@ -51,3 +62,10 @@ class DashboardAdmin(admin.ModelAdmin):
         if not request.user.is_superuser:
             readonly_fields.append("owned_by")
         return readonly_fields
+
+    def get_queryset(self, request):
+        if request.user.is_superuser:
+            # Superusers should be able to see all dashboards.
+            return super().get_queryset(request)
+        # Otherwise, show only the dashboards the user has edit access to.
+        return Dashboard.get_editable_by_user(request.user)
